from yta_positioning.coordinate import NORMALIZATION_MAX_VALUE, NORMALIZATION_MIN_VALUE
# I want these 2 below to be exported
from yta_positioning.validation import validate_size, validate_position
from yta_validation import PythonValidator
from yta_validation.number import NumberValidator
from yta_validation.parameter import ParameterValidator
from typing import Union


# TODO: These 2 methods below are similar to the ones in
# yta_multimedia\video\edition\effect\moviepy\position\objects\coordinate.py
# TODO: Maybe wrap in a 'VideoParametersValidator'
# class (?)
def is_size_valid(
    size: Union[tuple, list]
):
    """
    Check if the provided 'size' is a valid
    value or not, which means that is a tuple,
    list or array of 2 elements that are
    between the expected values.
    """
    return PythonValidator.is_numeric_tuple_or_list_or_array_of_2_elements_between_values(size, 1, NORMALIZATION_MAX_VALUE, 1, NORMALIZATION_MAX_VALUE)

def is_duration_valid(
    duration: float
) -> bool:
    """
    Check if the provided 'duration' is a valid
    duration, which means a positive number.
    """
    # TODO: We could maybe accept 'fps' parameter to check
    # if 'duration' is a multiple
    return NumberValidator.is_positive_number('duration', duration, do_include_zero = False)

def validate_duration(
    duration: float
) -> None:
    """
    Check if the provided 'duration' is
    valid to build a moviepy video clip
    or raises an exception if not.
    """
    # TODO: We could maybe accept 'fps' parameter to check
    # if 'duration' is a multiple
    ParameterValidator.validate_mandatory_positive_number('duration', duration, do_include_zero = False)

def is_fps_valid(
    fps: float
) -> bool:
    """
    Check if the provided 'fps' is valid to
    build a moviepy video clip, which must
    be a positive number.
    """
    return NumberValidator.is_positive_number('fps', fps, do_include_zero = False)

def validate_fps(
    fps: float
):
    """
    Check if the provided 'fps' is valid
    to build a moviepy video clip or
    raises an exception if not.
    """
    ParameterValidator.validate_mandatory_positive_number('fps', fps, do_include_zero = False)

def is_opacity_valid(
    opacity: float
) -> bool:
    """
    Check if the provided 'opacity' is 
    valid to build a moviepy video clip,
    which must be a value between 0.0 
    and 1.0.
    """
    return NumberValidator.is_number_between(opacity, 0.0, 1.0)

def validate_opacity(
    opacity: float
):
    """
    Check if the provided 'opacity' is
    valid to build a moviepy mask
    video clip or raises an exception
    if not.
    """
    ParameterValidator.validate_mandatory_number_between('opacity', opacity, 0.0, 1.0)
    
def is_position_valid(
    position: Union[list, tuple]
):
    """
    It is a list or tuple of 2 values which are between
    our min and max normalization values.
    """
    return PythonValidator.is_numeric_tuple_or_list_or_array_of_2_elements_between_values(position, NORMALIZATION_MIN_VALUE, NORMALIZATION_MAX_VALUE, NORMALIZATION_MIN_VALUE, NORMALIZATION_MAX_VALUE)



from yta_validation.parameter import ParameterValidator
from yta_file.filename.handler import FilenameHandler, FileType
from yta_file.handler import FileHandler
from yta_programming.decorators.requires_dependency import requires_dependency


# TODO: Can I avoid the use of opencv or
# implement other methods just in case I
# have the library (?)
@requires_dependency('cv2', 'yta_video_utils', 'opencv-python')
def get_video_size(
    video_filename: str
):
    """
    Get a tuple (w, h) containing the size of
    the provided 'video_filename' if it is a
    valid video file, or raise an exception if
    not.
    """
    import cv2
    
    # TODO: Refactor this method to allow any kind
    # of video parameter and I detect dynamically
    # if str to use this below, or clip to use moviepy
    ParameterValidator.validate_mandatory_string('video_filename', video_filename)
    
    # TODO: This validation is not parsing the content
    if (
        not FileHandler.is_file(video_filename) or
        not FilenameHandler.is_of_type(video_filename, FileType.VIDEO)
    ):
        raise Exception('The provided "video_filename" is not a valid video file name.')
    
    # TODO: I don't know where did this method
    # 'file_is_video_file' go when refactoring X)
    # if not FileValidator.file_is_video_file(video_filename):
    #     raise Exception('The provided "video_filename" is not a valid video file name.')

    v = cv2.VideoCapture(video_filename)

    return (
        int(v.get(cv2.CAP_PROP_FRAME_WIDTH)),
        int(v.get(cv2.CAP_PROP_FRAME_HEIGHT))
    )

__all__ = [
    get_video_size
]

